package com.rjconsultores.ventaboletos.web.gui.controladores.configuracioneccomerciales;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.FechamentoParamptovta;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.service.FechamentoParamptovtaService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;

@Controller("busquedaFechamentoParamptovtaController")
@Scope("prototype")
public class BusquedaFechamentoParamptovtaController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;

	@Autowired
	private FechamentoParamptovtaService fechamentoParamptovtaService;
	
	@Autowired
    private transient PagedListWrapper<FechamentoParamptovta> plwParams;
	private Paging pagingParams;
	
	private MyListbox paramsList;
	
	private Combobox cmbEmpresa;
	private List<Empresa> lsEmpresas;
	
	private Combobox cmbPuntoventa;
	private List<PuntoVenta> lsPuntoventa;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {		
		
		lsEmpresas = UsuarioLogado.getUsuarioLogado().getEmpresa();
		setLsPuntoventa(new ArrayList<PuntoVenta>());
		
		super.doAfterCompose(comp);
		
		paramsList.setItemRenderer(new FechamentoParamptovtaListItemRenderer());
		paramsList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				FechamentoParamptovta params = (FechamentoParamptovta) paramsList.getSelected();
				visualizaParams(params);
			}
		});
		
		actualizaParamsList();
	}

	
	private void visualizaParams(FechamentoParamptovta _params){
		
		if (_params == null)
			return;
		
		Map<String,Object> args = new HashMap<String,Object>();
		args.put("fechamentoparamptovtaId", _params.getFechamentoparamptovtaId());
		args.put("paramsList", paramsList);

		openWindow("/gui/configuraciones_comerciales/editarFechamentoParamptovta.zul",
				Labels.getLabel("editarFechamentoParamptovtaController.window.title"), args, MODAL);
		
	}
	
	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		actualizaParamsList();
	}

	public void onClick$btnRefresh(Event ev) {
		actualizaParamsList();
	}

	public void onClick$btnNovo(Event ev) {
		FechamentoParamptovta FechamentoParamptovta = new FechamentoParamptovta();
		FechamentoParamptovta.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		FechamentoParamptovta.setActivo(Boolean.TRUE);
		visualizaParams(FechamentoParamptovta);
	}
	
	private void actualizaParamsList() {
		HibernateSearchObject<FechamentoParamptovta> paramsBusqueda =
				new HibernateSearchObject<FechamentoParamptovta>(FechamentoParamptovta.class, pagingParams.getPageSize());

		paramsBusqueda.addFilterEqual("activo", true);
		
		Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();		
		if (itemEmpresa != null) {
			Empresa empresa = (Empresa) itemEmpresa.getValue();
			paramsBusqueda.addFilterEqual("empresa", empresa);
		} else {
			paramsBusqueda.addFilterIn("empresa", lsEmpresas);
		}
		
		Comboitem itemPuntoventa = cmbPuntoventa.getSelectedItem();		
		if (itemPuntoventa != null) {
			PuntoVenta puntoventa = (PuntoVenta) itemPuntoventa.getValue();
			paramsBusqueda.addFilterEqual("puntoventa", puntoventa);
		} else {
			paramsBusqueda.addFilterIn("puntoventa", lsPuntoventa);
		}
		
		
		paramsBusqueda.addSortDesc("fechamentoparamptovtaId");

		plwParams.init(paramsBusqueda, paramsList, pagingParams);

		if (paramsList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaFechamentoParamptovtaController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}


	public MyListbox getParamsList() {
		return paramsList;
	}


	public void setParamsList(MyListbox paramsList) {
		this.paramsList = paramsList;
	}


	public Combobox getCmbEmpresa() {
		return cmbEmpresa;
	}


	public void setCmbEmpresa(Combobox cmbEmpresa) {
		this.cmbEmpresa = cmbEmpresa;
	}


	public List<Empresa> getLsEmpresas() {
		return lsEmpresas;
	}


	public void setLsEmpresas(List<Empresa> lsEmpresas) {
		this.lsEmpresas = lsEmpresas;
	}
	
	public List<PuntoVenta> getLsPuntoventa() {
		return lsPuntoventa;
	}


	public void setLsPuntoventa(List<PuntoVenta> lsPuntoventa) {
		this.lsPuntoventa = lsPuntoventa;
	}
}
