package com.rjconsultores.ventaboletos.web.utilerias;

import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.WrongValueException;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zk.ui.event.InputEvent;
import org.zkoss.zkplus.databind.BindingListModel;
import org.zkoss.zkplus.databind.BindingListModelList;
import org.zkoss.zkplus.spring.SpringUtil;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.service.EmpresaService;

public class MyComboboxEmpresa extends Combobox {

	private static final long serialVersionUID = 1L;
	
	private static Logger log = LogManager.getLogger(MyComboboxEmpresa.class);
	public final static int minLength = 2;
	private EmpresaService empresaService;
	private List<Empresa> lsEmpresa;
	private Empresa initialValue;
	private Integer indiceSelected = null;
	private boolean sinTodos = false;
	private Integer ID_TODOS = -1;

	public MyComboboxEmpresa() {
		super();

		empresaService = (EmpresaService) SpringUtil.getBean("empresaService");
		lsEmpresa = new ArrayList<Empresa>();

		this.setAutodrop(false);
		this.setAutocomplete(false);
		

		this.addEventListener("onOK", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				String strEmpresa = MyComboboxEmpresa.this.getText().toUpperCase();
				if (strEmpresa.length() < MyComboboxEmpresa.minLength) {
					return;
				}
				if (!strEmpresa.isEmpty()) {
					lsEmpresa = empresaService.buscaLike(strEmpresa);

					if (sinTodos) {
						log.debug("Sin todos");
						Empresa e = empresaService.obtenerID(ID_TODOS);
						lsEmpresa.remove(e);
					}

					BindingListModel listModelParada = new BindingListModelList(lsEmpresa, true);
					MyComboboxEmpresa.this.setModel(listModelParada);
					setIndiceSelected(null);
					if (!lsEmpresa.isEmpty()) {
						setIndiceSelected(0);
					}

					MyComboboxEmpresa.this.open();
				} else {
					lsEmpresa.clear();

					BindingListModel listModelEmpresa = new BindingListModelList(lsEmpresa, true);
					MyComboboxEmpresa.this.setModel(listModelEmpresa);
				}
			}
		});

		this.addEventListener("onChanging", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				InputEvent ev = (InputEvent) event;
				String strEmpresa = ev.getValue();
				if (strEmpresa.length() < 2) {
					lsEmpresa.clear();

					BindingListModel listModelEmpresa = new BindingListModelList(lsEmpresa, true);
					MyComboboxEmpresa.this.setModel(listModelEmpresa);

					MyComboboxEmpresa.this.close();
				}
			}
		});
	}

	public Empresa getInitialValue() {
		return initialValue;
	}

	public void setInitialValue(Empresa initialValue) {
		if (initialValue == null) {
			return;
		}
		List<Empresa> ls = new ArrayList<Empresa>();
		ls.add(initialValue);

		this.setModel(new BindingListModelList(ls, false));
		this.setText(initialValue.getNombempresa());
	}

	/**
	 * 
	 * @param checaBusqueda
	 * @throws WrongValueException
	 */
	public String getValue(boolean checaBusqueda) throws WrongValueException {
		if (checaBusqueda) {
			if (this.getSelectedItem() == null) {
				throw new WrongValueException(this, Labels.getLabel("MSG.Error.combobox.hacerBusqueda"));
			}
		}

		return super.getValue();
	}

	public boolean isSinTodos() {
		return sinTodos;
	}

	public void setSinTodos(boolean sinTodos) {
		this.sinTodos = sinTodos;
	}

	public void setComboItemByEmpresa(Empresa empresa) {

		List<Empresa> ls = new ArrayList<Empresa>();
		ls.add(empresa);

		this.setModel(new BindingListModelList(ls, false));
		this.setText(empresa.getNombempresa());
	}
	
	public Comboitem getItemPorId(Integer id) {
		for (Empresa emp : lsEmpresa) {
			if(emp.getEmpresaId().equals(id)) {
				Comboitem ciEmpresa = new Comboitem(emp.toString());
				ciEmpresa.setAttribute("value", emp);
				ciEmpresa.setValue(emp);
				return ciEmpresa;
			}
		}
		return null;
	}

	public Integer getIndiceSelected() {
		return indiceSelected;
	}

	public void setIndiceSelected(Integer indiceSelected) {
		this.indiceSelected = indiceSelected;
	}

	public Empresa getSelectedEmpresa() {
		return super.getSelectedItem() != null ? (Empresa) super.getSelectedItem().getValue():null;
	}

}
